# 倒入必要的库
import numpy as np
import matplotlib.pyplot as plt
# 从 scipy.ndimage 库中导入 gaussian_filter 函数, 这个函数是用来平滑数据的
from scipy.ndimage import gaussian_filter


# 定义一个画图函数，方便调用，函数跟数学里的函数一样，是一个黑盒子，输入一些东西，输出一些东西，在这里我们输入一个 ax 对象和一个整数 n，输出一个态密度图，ax 对象就是表示画在哪一幅图里，n 表示读取哪一个数据文件
def plot(ax, n):
	'''
    ax: matplotlib.pyplot.Axes 对象
    n: int, 数据文件编号
    '''
	# 使用 numpy 读取数据文件
	data = np.loadtxt(f'../data/dos/{n}_out.dat')
	# 定义一些所需的数据在列表里，方便后面调用
	# 储存轨道信息
	orbitals = ['Si $p$', 'Fe $d$']
	# 储存颜色
	color = ['r', 'b', 'brown', 'y']
	# 储存吸附能
	ads = [-1.10, -1.34, -1.80, -1.22]
	
	for i in range(1, 3):
		# 创建一个循环，循环次数是 1 到 3，也就是 1 和 2，用以画两个轨道的态密度
		# 画图，data[:, 0] 是能量，data[:, 2 *i - 1] 是态密度，gaussian_filter 是平滑数据，sigma 是平滑参数，越大越平滑，label 是图例，alpha 是透明度，color 是颜色，下面两行第一行画自旋向上，第二行画自旋向下
		ax.plot(data[:, 0], gaussian_filter(data[:, 2 * i - 1], sigma=2), label=orbitals[i-1], alpha=0.3,
		        color=color[i])
		ax.plot(data[:, 0], gaussian_filter(data[:, 2 * i], sigma=2), alpha=0.3, color=color[i])
		# 画辅助线 x=0 和 y=0
		ax.axvline(x=0, color='gray', linestyle='--', alpha=0.6)
		ax.axhline(y=0, color='gray', linestyle='--', alpha=0.6)
		# 设置坐标轴范围
		ax.set_xlim(-3, 3)
		ax.set_ylim(-2, 2)
		# 添加文本，显示吸附能和序列号，前面的数字是位置，第三个是显示的文本，transform 表示按比例放置在 ax 中，ha 是水平对齐方式，va 是垂直对齐方式
		ax.text(0.95, 0.95, f'{n}', fontsize=12, transform=ax.transAxes, ha='right', va='top')
		ax.text(0.95, 0.88, '$E_\mathrm{^*CO_2}= ' + f'{ads[int(n) - 3]:.2f}$ eV', fontsize=12, transform=ax.transAxes,
		        ha='right', va='top')


	# 设置坐标轴标签，fontsize 是字体大小
	ax.set_xlabel(r'Energy (eV)', fontsize=12)
	ax.set_ylabel(r'DOS (states $\mathrm{Hartee^{-1}\cdot u.c.^{-1}}$)', fontsize=12)
	# 显示图例
	ax.legend(loc='lower left', fontsize=10, frameon=False)

# 主函数，程序的入口
if __name__ == "__main__":
	# 设置字体
	# 启用 latex 渲染
	plt.rcParams["text.usetex"] = True
	# 导入一些 latex 包
	plt.rcParams["text.latex.preamble"] = r"\usepackage{amsmath} \usepackage{amssymb} \usepackage{bm}"
	# 设置字体
	plt.rcParams['font.family'] = 'serif'
	plt.rcParams['font.serif'] = ['Times New Roman']
	
	# 创建一个画布和一个坐标轴对象, figsize 是画布大小, 单位是英寸
	fig, ax = plt.subplots(figsize=(6, 4))
	
	# 调用我们上面定义的画图函数，画一幅图，画在 ax 上，数据文件是 3_out.dat
	plot(ax, 3)
	
	# 保存图片，第一个参数是保存的名字和路径，matplotlib 会根据后缀自动生成对应格式的图片，常见的包括.png, .jpg等都支持bbox_inches 是保存图片的边界，tight 表示紧凑保存，dpi 是指定位图分辨率，矢量图不需要设置，建议在文章中插入矢量图，比如 .pdf，不会失真。
	plt.savefig(f'../figure/{3}.pdf', bbox_inches='tight', dpi=330)